@@ -1,3 +1,5 @@
+#include <assert.h>
+
 struct ksmap {
   int keysym;
   unsigned int ucs;
@@ -5,6 +7,28 @@ struct ksmap {
 
 #include "kstoucs_map.c"
 
+/* Binary search through `kstoucs_map'.  */
+static unsigned int
+find_ucs (int keysym, struct ksmap *first, struct ksmap *last)
+{
+  struct ksmap *middle = first + (last - first) / 2;
+
+  assert (first <= last);
+
+  if (middle->keysym == keysym)
+    return middle->ucs; /* base case: needle found. */
+  else if (first == last) /* empty search space */
+    return 0;
+  /* recursive cases: halve search space. */
+  else if (middle->keysym < keysym)
+    return find_ucs (keysym, middle+1, last);
+  else if (middle->keysym > keysym)
+    /* don't remove middle from the range to compensate
+       for rounding down in it's calculation */
+    return find_ucs (keysym, first, middle);
+  return 0;
+}
+
 unsigned int
 KeySymToUcs4 (int keysym)
 {
@@ -23,23 +47,6 @@ unsigned int doit (int keysym)
   if ((keysym & 0xff000000) == 0x01000000)
     return (keysym & 0x00ffffff);
 
-  unsigned int
-  find_ucs (int keysym, struct ksmap *first, struct ksmap *last)
-  {
-    struct ksmap *middle = first + (last - first) / 2;
-
-    if (middle->keysym == keysym)
-      return middle->ucs; /* base case: needle found. */
-    else if (middle == first && middle == last)
-      return 0; /* base case: empty search space. */
-    /* recursive cases: halve search space. */
-    else if (middle->keysym < keysym)
-      return find_ucs (keysym, middle+1, last);
-    else if (middle->keysym > keysym)
-      return find_ucs (keysym, first, middle-1);
-    return 0;
-  }
-
   #define NUM_KEYSYMS (sizeof kstoucs_map / sizeof(struct ksmap))
   return find_ucs(keysym, &kstoucs_map[0], &kstoucs_map[NUM_KEYSYMS - 1]);
 #ifdef XKB_DEBUG
