@@ -1,5 +1,6 @@
 /* libdiskfs implementation of fs.defs:dir_lookup
-   Copyright (C) 1992, 1993, 1994, 1995, 1996, 1997 Free Software Foundation
+   Copyright (C) 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001,
+     2002, 2008, 2013, 2014 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -25,10 +26,6 @@
 #include "priv.h"
 #include "fs_S.h"
 
-/* XXX - Temporary hack; this belongs in a header file, probably types.h. */
-#define major(x) ((int)(((unsigned) (x) >> 8) & 0xff))
-#define minor(x) ((int)((x) & 0xff))
-
 /* Implement dir_lookup as described in <hurd/fs.defs>. */
 kern_return_t
 diskfs_S_dir_lookup (struct protid *dircred,
@@ -44,8 +41,9 @@ diskfs_S_dir_lookup (struct protid *dircred,
   struct node *np;
   int nsymlink = 0;
   char *nextname;
+  char *relpath;
   int nextnamelen;
-  error_t error = 0;
+  error_t err = 0;
   char *pathbuf = 0;
   int pathbuflen = 0;
   int newnamelen;
@@ -54,9 +52,10 @@ diskfs_S_dir_lookup (struct protid *dircred,
   int newnode = 0;
   struct dirstat *ds = 0;
   int mustbedir = 0;
-  int amt;
+  size_t amt;
   int type;
-  struct protid *newpi;
+  struct protid *newpi = 0;
+  struct peropen *newpo = 0;
 
   if (!dircred)
     return EOPNOTSUPP;
@@ -70,25 +69,32 @@ diskfs_S_dir_lookup (struct protid *dircred,
   while (path[0] == '/')
     path++;
 
+  /* Preserve the path relative to diruser->po->path.  */
+  relpath = strdup (path);
+  if (! relpath)
+    return ENOMEM;
+
+  /* Keep a pointer to the start of the path for length
+     calculations.  */
+  char *path_start = path;
+
   *returned_port_poly = MACH_MSG_TYPE_MAKE_SEND;
   *retry = FS_RETRY_NORMAL;
   retryname[0] = '\0';
 
   if (path[0] == '\0')
     {
-      mustbedir = 1;
-
       /* Set things up in the state expected by the code from gotit: on. */
       dnp = 0;
       np = dircred->po->np;
-      mutex_lock (&np->lock);
+      pthread_mutex_lock (&np->lock);
       diskfs_nref (np);
       goto gotit;
     }
 
   dnp = dircred->po->np;
 
-  mutex_lock (&dnp->lock);
+  pthread_mutex_lock (&dnp->lock);
   np = 0;
 
   diskfs_nref (dnp);		/* acquire a reference for later diskfs_nput */
@@ -96,7 +102,7 @@ diskfs_S_dir_lookup (struct protid *dircred,
   do
     {
       assert (!lastcomp);
-      
+
       /* Find the name of the next pathname component */
       nextname = index (path, '/');
 
@@ -118,64 +124,79 @@ diskfs_S_dir_lookup (struct protid *dircred,
 	}
       else
 	lastcomp = 1;
-	  
+
       np = 0;
 
       /* diskfs_lookup the next pathname component */
       if (lastcomp && create)
 	{
-	  assert (!ds);
-	  ds = alloca (diskfs_dirstat_size);
-	  error = diskfs_lookup (dnp, path, CREATE, &np, ds, dircred);
+	  if (!ds)
+	    ds = alloca (diskfs_dirstat_size);
+	  err = diskfs_lookup (dnp, path, CREATE, &np, ds, dircred);
 	}
       else
-	error = diskfs_lookup (dnp, path, LOOKUP, &np, 0, dircred);
+	err = diskfs_lookup (dnp, path, LOOKUP, &np, 0, dircred);
 
-      if (lastcomp && create && excl && (!error || error == EAGAIN))
-	error = EEXIST;
+      if (lastcomp && create && excl && (!err || err == EAGAIN))
+	err = EEXIST;
 
       /* If we get an error we're done */
-      if (error == EAGAIN)
-	if (dnp == dircred->po->shadow_root)
-	  /* We're at the root of a shadow tree.  */
-	  {
-	    *retry = FS_RETRY_REAUTH;
-	    *returned_port = dircred->po->shadow_root_parent;
-	    *returned_port_poly = MACH_MSG_TYPE_COPY_SEND;
-	    if (! lastcomp)
-	      strcpy (retryname, nextname);
-	    error = 0;
-	    goto out;
-	  }
-	else if (dircred->po->root_parent != MACH_PORT_NULL)
-	  /* We're at a real translator root; even if DIRCRED->po has a
-	     shadow root, we can get here if its in a directory that was
-	     renamed out from under it...  */
-	  {
-	    *retry = FS_RETRY_REAUTH;
-	    *returned_port = dircred->po->root_parent;
-	    *returned_port_poly = MACH_MSG_TYPE_COPY_SEND;
-	    if (!lastcomp)
-	      strcpy (retryname, nextname);
-	    error = 0;
-	    goto out;
-	  }
-	else
-	  /* We're at a REAL root, as in there's no way up from here.  */
-	  {
-	    error = 0;
-	    np = dnp;
-	    diskfs_nref (np);
-	  }
+      if (err == EAGAIN)
+	{
+	  if (dnp == dircred->po->shadow_root)
+	    /* We're at the root of a shadow tree.  */
+	    {
+	      if (dircred->po->shadow_root_parent == MACH_PORT_NULL)
+		{
+		  /* This is a shadow root with no parent, meaning
+		     we should treat it as a virtual root disconnected
+		  from its real .. directory.  */
+		  err = 0;
+		  np = dnp;
+		  diskfs_nref (np);
+		}
+	      else
+		{
+		  /* Punt the client up to the shadow root parent.  */
+		  *retry = FS_RETRY_REAUTH;
+		  *returned_port = dircred->po->shadow_root_parent;
+		  *returned_port_poly = MACH_MSG_TYPE_COPY_SEND;
+		  if (! lastcomp)
+		    strcpy (retryname, nextname);
+		  err = 0;
+		  goto out;
+		}
+	    }
+	  else if (dircred->po->root_parent != MACH_PORT_NULL)
+	    /* We're at a real translator root; even if DIRCRED->po has a
+	       shadow root, we can get here if its in a directory that was
+	    renamed out from under it...  */
+	    {
+	      *retry = FS_RETRY_REAUTH;
+	      *returned_port = dircred->po->root_parent;
+	      *returned_port_poly = MACH_MSG_TYPE_COPY_SEND;
+	      if (!lastcomp)
+		strcpy (retryname, nextname);
+	      err = 0;
+	      goto out;
+	    }
+	  else
+	    /* We're at a REAL root, as in there's no way up from here.  */
+	    {
+	      err = 0;
+	      np = dnp;
+	      diskfs_nref (np);
+	    }
+	}
 
       /* Create the new node if necessary */
       if (lastcomp && create)
 	{
-	  if (error == ENOENT)
+	  if (err == ENOENT)
 	    {
-	      mode &= ~(S_IFMT | S_ISPARE | S_ISVTX);
+	      mode &= ~(S_IFMT | S_ISPARE | S_ISVTX | S_ITRANS);
 	      mode |= S_IFREG;
-	      error = diskfs_create_node (dnp, path, mode, &np, dircred, ds);
+	      err = diskfs_create_node (dnp, path, mode, &np, dircred, ds);
 	      if (diskfs_synchronous)
 		{
 		  diskfs_file_update (dnp, 1);
@@ -186,8 +207,8 @@ diskfs_S_dir_lookup (struct protid *dircred,
 	  else
 	    diskfs_drop_dirstat (dnp, ds);
 	}
-      
-      if (error)
+
+      if (err)
 	goto out;
 
       /* If this is translated, start the translator (if necessary)
@@ -200,12 +221,13 @@ diskfs_S_dir_lookup (struct protid *dircred,
 	      || fshelp_translated (&np->transbox)))
 	{
 	  mach_port_t dirport;
-	  
+	  struct iouser *user;
+
 	  /* A callback function for short-circuited translators.
 	     Symlink & ifsock are handled elsewhere.  */
 	  error_t short_circuited_callback1 (void *cookie1, void *cookie2,
 					     uid_t *uid, gid_t *gid,
-					     char **argz, int *argz_len)
+					     char **argz, size_t *argz_len)
 	    {
 	      struct node *node = cookie1;
 
@@ -238,22 +260,35 @@ diskfs_S_dir_lookup (struct protid *dircred,
 
 	  /* Create an unauthenticated port for DNP, and then
 	     unlock it. */
-	  error = 
-	    diskfs_create_protid (diskfs_make_peropen (dnp, 0, dircred->po),
-				  iohelp_create_iouser (make_idvec (), 
-							make_idvec ()),
-				  &newpi);
-	  if (error)
+	  err = iohelp_create_empty_iouser (&user);
+	  if (! err)
+	    {
+	      err = diskfs_make_peropen (dnp, 0, dircred->po, &newpo);
+	      if (! err)
+		{
+		  err = diskfs_create_protid (newpo, user, &newpi);
+		  if (! err)
+		    newpo = 0;
+		}
+
+	      iohelp_free_iouser (user);
+	    }
+
+	  if (err)
 	    goto out;
 
-	  dirport = ports_get_right (newpi);
-	  mach_port_insert_right (mach_task_self (), dirport, dirport,
-				  MACH_MSG_TYPE_MAKE_SEND);
-	  ports_port_deref (newpi);
+	  dirport = ports_get_send_right (newpi);
 	  if (np != dnp)
-	    mutex_unlock (&dnp->lock);
+	    pthread_mutex_unlock (&dnp->lock);
 
-	  error = fshelp_fetch_root (&np->transbox, dircred->po,
+	  /* Check if an active translator is currently running.  If
+	     not, fshelp_fetch_root will start one.  In that case, we
+	     need to register it in the list of active
+	     translators.  */
+	  boolean_t register_translator =
+	    np->transbox.active == MACH_PORT_NULL;
+
+	  err = fshelp_fetch_root (&np->transbox, dircred->po,
 				     dirport, dircred->user,
 				     lastcomp ? flags : 0,
 				     ((np->dn_stat.st_mode & S_IPTRANS)
@@ -266,91 +301,149 @@ diskfs_S_dir_lookup (struct protid *dircred,
 	     deallocate our send right.  */
 	  mach_port_deallocate (mach_task_self (), dirport);
 
-	  if (error != ENOENT)
+	  if (err != ENOENT)
 	    {
-	      diskfs_nrele (dnp);
-	      diskfs_nput (np);
 	      *returned_port_poly = MACH_MSG_TYPE_MOVE_SEND;
-	      if (!lastcomp && !error)
+	      if (!lastcomp && !err)
 		{
-		  strcat (retryname, "/");
-		  strcat (retryname, nextname);
+		  char *end = strchr (retryname, '\0');
+		  *end++ = '/';
+		  strcpy (end, nextname);
 		}
-	      return error;
+
+	      if (register_translator)
+		{
+		  char *translator_path = strdupa (relpath);
+		  char *complete_path;
+		  if (nextname != NULL)
+		    {
+		      /* This was not the last path component.
+			 NEXTNAME points to the next component, locate
+			 the end of the current component and use it
+			 to trim TRANSLATOR_PATH.  */
+		      char *end = nextname;
+		      while (*end != 0)
+			end--;
+		      translator_path[end - path_start] = '\0';
+		    }
+
+		  if (dircred->po->path == NULL || !strcmp (dircred->po->path,"."))
+		      /* dircred is the root directory.  */
+		      complete_path = translator_path;
+		  else
+		      asprintf (&complete_path, "%s/%s", dircred->po->path, translator_path);
+
+		  err = fshelp_set_active_translator (&newpi->pi,
+							complete_path,
+							np->transbox.active);
+		  if (complete_path != translator_path)
+		    free(complete_path);
+		  if (err)
+		    goto out;
+		}
+
+	      goto out;
 	    }
 
+	  ports_port_deref (newpi);
+	  newpi = NULL;
+
 	  /* ENOENT means there was a hiccup, and the translator
 	     vanished while NP was unlocked inside fshelp_fetch_root.
 	     Reacquire the locks, and continue as normal. */
-	  error = 0;
+	  err = 0;
 	  if (np != dnp)
 	    {
 	      if (!strcmp (path, ".."))
-		mutex_lock (&dnp->lock);
+		pthread_mutex_lock (&dnp->lock);
 	      else
 		{
-		  mutex_unlock (&np->lock);
-		  mutex_lock (&dnp->lock);
-		  mutex_lock (&np->lock);
+		  if (pthread_mutex_trylock (&dnp->lock))
+		    {
+		      pthread_mutex_unlock (&np->lock);
+		      pthread_mutex_lock (&dnp->lock);
+		      pthread_mutex_lock (&np->lock);
+		    }
 		}
 	    }
 	}
-      
+
       if (S_ISLNK (np->dn_stat.st_mode)
-	  && !(lastcomp && (flags & (O_NOLINK|O_NOTRANS))))
+	  && (!lastcomp
+	      || mustbedir	/* "foo/" must see that foo points to a dir */
+	      || !(flags & (O_NOLINK|O_NOTRANS))))
 	{
 	  /* Handle symlink interpretation */
 
 	  if (nsymlink++ > diskfs_maxsymlinks)
 	    {
-	      error = ELOOP;
+	      err = ELOOP;
 	      goto out;
 	    }
-	      
+
 	  nextnamelen = nextname ? strlen (nextname) + 1 : 0;
-	  newnamelen = nextnamelen + np->dn_stat.st_size + 1;
+	  newnamelen = nextnamelen + np->dn_stat.st_size + 1 + 1;
 	  if (pathbuflen < newnamelen)
 	    {
 	      pathbuf = alloca (newnamelen);
 	      pathbuflen = newnamelen;
 	    }
-	      
-	  if (diskfs_read_symlink_hook)
-	    error = (*diskfs_read_symlink_hook)(np, pathbuf);
-	  if (!diskfs_read_symlink_hook || error == EINVAL)
-	    error = diskfs_node_rdwr (np, pathbuf, 
-				      0, np->dn_stat.st_size, 0, 
-				      dircred, &amt);
-	  if (error)
-	    goto out;
-	      
-	  if (nextname)
-	    {
-	      pathbuf[np->dn_stat.st_size] = '/';
-	      bcopy (nextname, pathbuf + np->dn_stat.st_size + 1, 
-		     nextnamelen - 1);
-	    }
-	  pathbuf[nextnamelen + np->dn_stat.st_size] = '\0';
 
-	  if (pathbuf[0] == '/')
+	  if (diskfs_read_symlink_hook)
+	    err = (*diskfs_read_symlink_hook)(np, pathbuf);
+	  if (!diskfs_read_symlink_hook || err == EINVAL)
 	    {
-	      /* Punt to the caller.  */
-	      *retry = FS_RETRY_MAGICAL;
-	      *returned_port = MACH_PORT_NULL;
-	      strcpy (retryname, pathbuf);
-	      goto out;
+	      err = diskfs_node_rdwr (np, pathbuf,
+					0, np->dn_stat.st_size, 0,
+					dircred, &amt);
+	      if (!err)
+		assert (amt == np->dn_stat.st_size);
 	    }
-	  
-	  path = pathbuf;
+	  if (err)
+	    goto out;
+
+	  if (np->dn_stat.st_size == 0)	/* symlink to "" */
+	    path = nextname;
+	  else
+	    {
+	      if (nextname)
+		{
+		  pathbuf[np->dn_stat.st_size] = '/';
+		  memcpy (pathbuf + np->dn_stat.st_size + 1,
+			  nextname, nextnamelen - 1);
+		}
+	      pathbuf[nextnamelen + np->dn_stat.st_size] = '\0';
+
+	      if (pathbuf[0] == '/')
+		{
+		  /* Punt to the caller.  */
+		  *retry = FS_RETRY_MAGICAL;
+		  *returned_port = MACH_PORT_NULL;
+		  memcpy (retryname, pathbuf,
+			  nextnamelen + np->dn_stat.st_size + 1);
+		  if (mustbedir)
+		    {
+		      retryname[nextnamelen + np->dn_stat.st_size] = '/';
+		      retryname[nextnamelen + np->dn_stat.st_size + 1] = '\0';
+		    }
+		  goto out;
+		}
+
+	      path = pathbuf;
+	    }
+
 	  if (lastcomp)
-	    {
-	      lastcomp = 0;
-	      /* Symlinks to nonexistent files aren't allowed to cause
-		 creation, so clear the flag here. */
-	      create = 0;
-	    }
+	    lastcomp = 0;
+
 	  diskfs_nput (np);
 	  np = 0;
+
+	  if (path == 0)	/* symlink to "" was the last component */
+	    {
+	      np = dnp;
+	      dnp = 0;
+	      break;
+	    }
 	}
       else
 	{
@@ -369,7 +462,7 @@ diskfs_S_dir_lookup (struct protid *dircred,
 	    dnp = 0;
 	}
     } while (path && *path);
-  
+
   /* At this point, np is the node to return.  If newnode is set, then
      we just created this node.  */
 
@@ -378,10 +471,10 @@ diskfs_S_dir_lookup (struct protid *dircred,
 
   if (mustbedir && type != S_IFDIR)
     {
-      error = ENOTDIR;
+      err = ENOTDIR;
       goto out;
     }
-  
+
   if (!newnode)
     /* Check permissions on existing nodes, but not new ones. */
     {
@@ -389,64 +482,89 @@ diskfs_S_dir_lookup (struct protid *dircred,
 	    type == S_IFIFO)
 	   && (flags & (O_READ|O_WRITE|O_EXEC)))
 	  || (type == S_IFLNK && (flags & (O_WRITE|O_EXEC))))
-	error = EOPNOTSUPP;
+	err = EACCES;
 
-      if (!error && (flags & O_READ))
-	error = fshelp_access (&np->dn_stat, S_IREAD, dircred->user);
+      if (!err && (flags & O_READ))
+	err = fshelp_access (&np->dn_stat, S_IREAD, dircred->user);
 
-      if (!error && (flags & O_EXEC))
-	error = fshelp_access (&np->dn_stat, S_IEXEC, dircred->user);
+      if (!err && (flags & O_EXEC))
+	err = fshelp_access (&np->dn_stat, S_IEXEC, dircred->user);
 
-      if (!error && (flags & O_WRITE))
+      if (!err && (flags & O_WRITE))
 	{
 	  if (type == S_IFDIR)
-	    error = EISDIR;
+	    err = EISDIR;
 	  else if (diskfs_check_readonly ())
-	    error = EROFS;
+	    err = EROFS;
 	  else
-	    error = fshelp_access (&np->dn_stat, S_IWRITE, dircred->user);
+	    err = fshelp_access (&np->dn_stat, S_IWRITE, dircred->user);
 	}
 
-      if (error)
+      if (err)
 	goto out;
     }
-    
-  if ((flags & O_NOATIME) 
+
+  if ((flags & O_NOATIME)
       && (fshelp_isowner (&np->dn_stat, dircred->user) == EPERM))
     flags &= ~O_NOATIME;
-      
-  error =
-    diskfs_create_protid (diskfs_make_peropen (np, 
-					       (flags &~OPENONLY_STATE_MODES), 
-					       dircred->po),
-			  dircred->user, &newpi);
 
-  if (! error)
+  err = diskfs_make_peropen (np, (flags &~OPENONLY_STATE_MODES),
+			       dircred->po, &newpo);
+
+  if (! err)
+    err = diskfs_create_protid (newpo, dircred->user, &newpi);
+
+  if (! err)
     {
+      newpo = 0;
       if (flags & O_EXLOCK)
-	error = fshelp_acquire_lock (&np->userlock, &newpi->po->lock_status,
+	err = fshelp_acquire_lock (&np->userlock, &newpi->po->lock_status,
 				     &np->lock, LOCK_EX);
       else if (flags & O_SHLOCK)
-	error = fshelp_acquire_lock (&np->userlock, &newpi->po->lock_status,
+	err = fshelp_acquire_lock (&np->userlock, &newpi->po->lock_status,
 				     &np->lock, LOCK_SH);
-      if (error)
-	ports_port_deref (newpi); /* Get rid of NEWPI.  */
     }
-  
-  if (! error)
+
+  if (! err)
     {
+      free (newpi->po->path);
+      if (dircred->po->path == NULL || !strcmp (dircred->po->path,"."))
+	{
+	  /* dircred is the root directory.  */
+	  newpi->po->path = relpath;
+	  relpath = NULL; /* Do not free relpath.  */
+	}
+      else
+	{
+	  newpi->po->path = NULL;
+	  asprintf (&newpi->po->path, "%s/%s", dircred->po->path, relpath);
+	}
+
+      if (! newpi->po->path)
+	err = errno;
+
       *returned_port = ports_get_right (newpi);
       ports_port_deref (newpi);
+      newpi = 0;
     }
-  
+
  out:
   if (np)
-    if (dnp == np)
-      diskfs_nrele (np);
-    else
-      diskfs_nput (np);
+    {
+      if (dnp == np)
+	diskfs_nrele (np);
+      else
+	diskfs_nput (np);
+    }
   if (dnp)
     diskfs_nput (dnp);
 
-  return error;
+  if (newpi)
+    ports_port_deref (newpi);
+  if (newpo)
+    diskfs_release_peropen (newpo);
+
+  free (relpath);
+
+  return err;
 }
