@@ -1,5 +1,5 @@
 /* libdiskfs implementation of fs.defs: dir_unlink
-   Copyright (C) 1992, 1993, 1994, 1995, 1996, 1997 Free Software Foundation
+   Copyright (C) 1992,93,94,95,96,97,2002 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -27,28 +27,28 @@ diskfs_S_dir_unlink (struct protid *dircred,
   struct node *dnp;
   struct node *np;
   struct dirstat *ds = alloca (diskfs_dirstat_size);
-  error_t error;
+  error_t err;
   mach_port_t control = MACH_PORT_NULL;
 
   if (!dircred)
     return EOPNOTSUPP;
-  
+
   dnp = dircred->po->np;
   if (diskfs_check_readonly ())
     return EROFS;
 
-  mutex_lock (&dnp->lock);
+  pthread_mutex_lock (&dnp->lock);
 
-  error = diskfs_lookup (dnp, name, REMOVE, &np, ds, dircred);
-  if (error == EAGAIN)
-    error = EISDIR;
-  if (error)
+  err = diskfs_lookup (dnp, name, REMOVE, &np, ds, dircred);
+  if (err == EAGAIN)
+    err = EPERM;	/* 1003.1-1996 5.5.1.4 */
+  if (err)
     {
       diskfs_drop_dirstat (dnp, ds);
-      mutex_unlock (&dnp->lock);
-      return error;
+      pthread_mutex_unlock (&dnp->lock);
+      return err;
     }
-  
+
   /* This isn't the BSD behavior, but it is Posix compliant and saves
      us on several race conditions.*/
   if (S_ISDIR(np->dn_stat.st_mode))
@@ -58,20 +58,20 @@ diskfs_S_dir_unlink (struct protid *dircred,
       else
 	diskfs_nput (np);
       diskfs_drop_dirstat (dnp, ds);
-      mutex_unlock (&dnp->lock);
-      return EISDIR;
+      pthread_mutex_unlock (&dnp->lock);
+      return EPERM;		/* 1003.1-1996 5.5.1.4 */
     }
-  
-  error = diskfs_dirremove (dnp, np, name, ds);
+
+  err = diskfs_dirremove (dnp, np, name, ds);
   if (diskfs_synchronous)
     diskfs_node_update (dnp, 1);
-  if (error)
+  if (err)
     {
       diskfs_nput (np);
-      mutex_unlock (&dnp->lock);
-      return error;
+      pthread_mutex_unlock (&dnp->lock);
+      return err;
     }
-      
+
   np->dn_stat.st_nlink--;
   np->dn_set_ctime = 1;
   if (diskfs_synchronous)
@@ -80,13 +80,13 @@ diskfs_S_dir_unlink (struct protid *dircred,
   if (np->dn_stat.st_nlink == 0)
     fshelp_fetch_control (&np->transbox, &control);
 
-  /* This check is necessary because we might get here on an error while 
+  /* This check is necessary because we might get here on an error while
      checking the mode on something which happens to be `.'. */
   if (np == dnp)
-    diskfs_nrele (np);	
+    diskfs_nrele (np);
   else
     diskfs_nput (np);
-  mutex_unlock (&dnp->lock);
+  pthread_mutex_unlock (&dnp->lock);
 
   if (control)
     {
@@ -94,5 +94,5 @@ diskfs_S_dir_unlink (struct protid *dircred,
       mach_port_deallocate (mach_task_self (), control);
     }
 
-  return error;
+  return err;
 }
