@@ -21,7 +21,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include <mach.h>
 #include <hurd.h>
-#include <cthreads.h>
+#include <pthread.h>
 #include <hurd/ports.h>
 #include "fshelp.h"
 #include <sys/file.h>
