@@ -1,5 +1,5 @@
 /* 
-   Copyright (C) 1995 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
    Written by Michael I. Bushnell, p/BSG.
 
    This file is part of the GNU Hurd.
@@ -29,10 +29,11 @@ netfs_S_file_set_size (struct protid *user,
   
   if (!user)
     return EOPNOTSUPP;
+  else if (size < 0)
+    return EINVAL;
   
-  mutex_lock (&user->po->np->lock);
-  err = netfs_attempt_set_size (user->credential, user->po->np, size);
-  mutex_unlock (&user->po->np->lock);
+  pthread_mutex_lock (&user->po->np->lock);
+  err = netfs_attempt_set_size (user->user, user->po->np, size);
+  pthread_mutex_unlock (&user->po->np->lock);
   return err;
 }
-
