@@ -1,5 +1,5 @@
-/* 
-   Copyright (C) 1995, 1996 Free Software Foundation, Inc.
+/*
+   Copyright (C) 1995,96,2000,01 Free Software Foundation, Inc.
    Written by Michael I. Bushnell, p/BSG.
 
    This file is part of the GNU Hurd.
@@ -24,31 +24,44 @@
 error_t
 netfs_S_io_reauthenticate (struct protid *user, mach_port_t rend_port)
 {
-  struct protid *newpi;
   error_t err;
+  struct protid *newpi;
   mach_port_t newright;
-  
+
   if (!user)
     return EOPNOTSUPP;
-  
-  mutex_lock (&user->po->np->lock);
-  newpi = netfs_make_protid (user->po, 0);
 
-  newright = ports_get_right (newpi);
-  err = mach_port_insert_right (mach_task_self (), newright, newright,
-				MACH_MSG_TYPE_MAKE_SEND);
-  assert_perror (err);
+  /* This routine must carefully ignore EINTR because we
+     are a simpleroutine, so callers won't know to restart. */
 
-  newpi->user = iohelp_reauth (netfs_auth_server_port, rend_port, newright, 1);
+  pthread_mutex_lock (&user->po->np->lock);
+  do
+    newpi = netfs_make_protid (user->po, 0);
+  while (! newpi && errno == EINTR);
+  if (! newpi)
+    {
+      pthread_mutex_unlock (&user->po->np->lock);
+      return errno;
+    }
 
-  mach_port_deallocate (mach_task_self (), rend_port);
+  newright = ports_get_send_right (newpi);
+  assert (newright != MACH_PORT_NULL);
+
+  /* Release the node lock while blocking on the auth server and client.  */
+  pthread_mutex_unlock (&user->po->np->lock);
+  err = iohelp_reauth (&newpi->user, netfs_auth_server_port, rend_port,
+		       newright, 1);
+  pthread_mutex_lock (&user->po->np->lock);
+
+  if (!err)
+    mach_port_deallocate (mach_task_self (), rend_port);
   mach_port_deallocate (mach_task_self (), newright);
 
   mach_port_move_member (mach_task_self (), newpi->pi.port_right,
 			 netfs_port_bucket->portset);
 
-  mutex_unlock (&user->po->np->lock);
+  pthread_mutex_unlock (&user->po->np->lock);
   ports_port_deref (newpi);
 
-  return 0;
+  return err;
 }
