@@ -43,8 +43,11 @@ ports_reallocate_from_external (void *portstruct, mach_port_t receive)
 			    MACH_PORT_RIGHT_RECEIVE, -1);
   assert_perror (err);
 
+  pthread_rwlock_wrlock (&_ports_htable_lock);
+  hurd_ihash_locp_remove (&_ports_htable, pi->ports_htable_entry);
   hurd_ihash_locp_remove (&pi->bucket->htable, pi->hentry);
-  
+  pthread_rwlock_unlock (&_ports_htable_lock);
+
   if ((pi->flags & PORT_HAS_SENDRIGHTS) && !stat.mps_srights)
     {
       dropref = 1;
@@ -53,16 +56,25 @@ ports_reallocate_from_external (void *portstruct, mach_port_t receive)
   else if (((pi->flags & PORT_HAS_SENDRIGHTS) == 0) && stat.mps_srights)
     {
       pi->flags |= PORT_HAS_SENDRIGHTS;
-      pi->refcnt++;
+      refcounts_ref (&pi->refcounts, NULL);
     }
   
   pi->port_right = receive;
   pi->cancel_threshold = 0;
   pi->mscount = stat.mps_mscount;
-  
-  hurd_ihash_add (&pi->bucket->htable, receive, pi);
+
+  pthread_rwlock_wrlock (&_ports_htable_lock);
+  err = hurd_ihash_add (&_ports_htable, receive, pi);
+  assert_perror (err);
+  err = hurd_ihash_add (&pi->bucket->htable, receive, pi);
+  pthread_rwlock_unlock (&_ports_htable_lock);
   pthread_mutex_unlock (&_ports_lock);
-  
+  assert_perror (err);
+
+  /* This is an optimization.  It may fail.  */
+  mach_port_set_protected_payload (mach_task_self (), pi->port_right,
+				   (unsigned long) pi);
+
   mach_port_move_member (mach_task_self (), receive, pi->bucket->portset);
   
   if (stat.mps_srights)
