@@ -1,7 +1,7 @@
 /* Implements the ps_fmt type, which describes how to output a user-readable
    version of a proc_stat.
 
-   Copyright (C) 1995, 1996, 1997 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996, 1997, 1998 Free Software Foundation, Inc.
 
    Written by Miles Bader <miles@gnu.ai.mit.edu>
 
@@ -68,6 +68,8 @@ _fmt_create (char *src, int posix, struct ps_fmt_specs *fmt_specs,
   src = new_fmt->src;
   while (*src != '\0')
     {
+      char *start = src;
+
       if (field - fields == fields_alloced)
 	/* Time to grow FIELDS to make room for more.  */
 	{
@@ -172,6 +174,19 @@ _fmt_create (char *src, int posix, struct ps_fmt_specs *fmt_specs,
 	    /* This field spec doesn't have a name, so use its flags fields
 	       to set the global ones, and skip it.  */
 	    {
+	      /* if we didn't use any chars, don't loop indefinitely */
+	      if (src == start)
+		{
+		  if (err_string)
+		    asprintf (err_string, "%s: Unknown format spec", src);
+
+		  FREE (new_fmt->src);
+		  FREE (new_fmt);
+		  FREE (fields);
+
+		  return EINVAL;
+		}
+
 	      global_clr_flags = clr_flags;
 	      global_inv_flags = inv_flags;
 	      continue;
@@ -194,7 +209,9 @@ _fmt_create (char *src, int posix, struct ps_fmt_specs *fmt_specs,
 		  while (*src != '\0' && *src != stop)
 		    src++;
 		}
-	      *src++ = '\0';	/* NUL terminhate NAME. */
+
+	      if (*src)
+	        *src++ = '\0';	/* NUL terminate NAME. */
 	    }
 	  else
 	    /* A gnu-style field spec: `NAME' or `NAME:TITLE'.  */
@@ -240,11 +257,13 @@ _fmt_create (char *src, int posix, struct ps_fmt_specs *fmt_specs,
 	    }
 
 	  if (! field->title)
-	    /* No explicit title specified in the fmt string.  */
-	    if (field->spec->title)
-	      field->title = field->spec->title; /* But the spec has one.  */
-	    else
-	      field->title = field->spec->name; /* Just use the field name.  */
+	    {
+	      /* No explicit title specified in the fmt string.  */
+	      if (field->spec->title)
+		field->title = field->spec->title; /* But the spec has one.  */
+	      else
+		field->title = field->spec->name; /* Just use field name.  */
+	    }
 
 	  /* Add FIELD's required pstat_flags to FMT's set */
 	  needs |= ps_getter_needs (ps_fmt_spec_getter (field->spec));
@@ -255,7 +274,7 @@ _fmt_create (char *src, int posix, struct ps_fmt_specs *fmt_specs,
 	    field->precision = field->spec->precision;
 
 	  field->flags = (field->spec->flags & ~clr_flags) ^ inv_flags;
-	  
+
 	  if (quoted_name && *src == '}')
 	    /* Skip optional trailing `}' after the spec name.  */
 	    src++;
@@ -324,7 +343,7 @@ ps_fmt_creation_error (char *src, int posix, struct ps_fmt_specs *fmt_specs,
 }
 
 /* Free FMT, and any resources it consumes.  */
-void 
+void
 ps_fmt_free (struct ps_fmt *fmt)
 {
   FREE (fmt->src);
@@ -366,8 +385,8 @@ ps_fmt_clone (struct ps_fmt *fmt, struct ps_fmt **copy)
   return 0;
 }
 
-/* Write an appropiate header line for FMT, containing the titles of all its
-   fields appropiately aligned with where the values would be printed, to
+/* Write an appropriate header line for FMT, containing the titles of all its
+   fields appropriately aligned with where the values would be printed, to
    STREAM (without a trailing newline).  If count is non-NULL, the total
    number number of characters output is added to the integer it points to.
    If any fatal error occurs, the error code is returned, otherwise 0.  */
@@ -457,9 +476,9 @@ ps_fmt_write_proc_stat (struct ps_fmt *fmt, struct proc_stat *ps, struct ps_stre
 }
 
 /* Remove those fields from FMT for which the function FN, when called on the
-   field, returns true.  Appropiate inter-field characters are also removed:
+   field, returns true.  Appropriate inter-field characters are also removed:
    those *following* deleted fields at the beginning of the fmt, and those
-   *preceeding* deleted fields *not* at the beginning. */
+   *preceding* deleted fields *not* at the beginning. */
 void
 ps_fmt_squash (struct ps_fmt *fmt, int (*fn)(struct ps_fmt_field *field))
 {
@@ -475,7 +494,7 @@ ps_fmt_squash (struct ps_fmt *fmt, int (*fn)(struct ps_fmt_field *field))
       {
 	/* Save the old prefix, in case we're deleting the first field,
 	   and need to prepend it to the next field.  */
-	const char *beg_pfx = field->pfx; 
+	const char *beg_pfx = field->pfx;
 	int beg_pfx_len = field->pfx_len;
 
 	nfields--;
@@ -536,8 +555,8 @@ ps_fmt_squash (struct ps_fmt *fmt, int (*fn)(struct ps_fmt_field *field))
 }
 
 /* Remove those fields from FMT which would need the proc_stat flags FLAGS.
-   Appropiate inter-field characters are also removed: those *following*
-   deleted fields at the beginning of the fmt, and those *preceeding* deleted
+   Appropriate inter-field characters are also removed: those *following*
+   deleted fields at the beginning of the fmt, and those *preceding* deleted
    fields *not* at the beginning.  */
 void
 ps_fmt_squash_flags (struct ps_fmt *fmt, ps_flags_t flags)
