@@ -1,5 +1,5 @@
-/* 
-   Copyright (C) 1994 Free Software Foundation
+/*
+   Copyright (C) 1994,2002 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -16,10 +16,10 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */
 
 #include "priv.h"
-#include "fs_S.h"
+#include "trivfs_fs_S.h"
 
 kern_return_t
-trivfs_S_file_chflags (struct trivfs_protid *cred, 
+trivfs_S_file_chflags (struct trivfs_protid *cred,
 		       mach_port_t reply, mach_msg_type_name_t reply_type,
 		       int flags)
 {
