@@ -1,26 +1,26 @@
 /*
-   Copyright (C) 1993, 1994, 1995, 1996 Free Software Foundation
+   Copyright (C) 1993,94,95,96,2000,01,02 Free Software Foundation, Inc.
 
-This file is part of the GNU Hurd.
+   This file is part of the GNU Hurd.
 
-The GNU Hurd is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
+   The GNU Hurd is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2, or (at your option)
+   any later version.
 
-The GNU Hurd is distributed in the hope that it will be useful, 
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+   The GNU Hurd is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with the GNU Hurd; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   You should have received a copy of the GNU General Public License
+   along with the GNU Hurd; see the file COPYING.  If not, write to
+   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* Written by Michael I. Bushnell.  */
 
 #include "priv.h"
-#include "io_S.h"
+#include "trivfs_io_S.h"
 #include <assert.h>
 #include <string.h>
 
@@ -41,36 +41,34 @@ trivfs_S_io_reauthenticate (struct trivfs_protid *cred,
   do
     err = ports_create_port_noinstall (cred->po->cntl->protid_class,
 				       cred->po->cntl->protid_bucket,
-				       sizeof (struct trivfs_protid), 
+				       sizeof (struct trivfs_protid),
 				       &newcred);
   while (err == EINTR);
   if (err)
     return err;
 
   auth = getauth ();
-  newright = ports_get_right (newcred);
-  err = mach_port_insert_right (mach_task_self (), newright, newright,
-				MACH_MSG_TYPE_MAKE_SEND);
-  assert_perror (err);
+  newright = ports_get_send_right (newcred);
+  assert (newright != MACH_PORT_NULL);
 
-  newcred->user = iohelp_reauth (auth, rendport, newright, 1);
+  err = iohelp_reauth (&newcred->user, auth, rendport, newright, 1);
+  if (!err)
+    mach_port_deallocate (mach_task_self (), rendport);
+  mach_port_deallocate (mach_task_self (), auth);
+  if (err)
+    return err;
+
+  mach_port_deallocate (mach_task_self (), newright);
   if (idvec_contains (newcred->user->uids, 0))
     newcred->isroot = 1;
 
-  mach_port_deallocate (mach_task_self (), rendport);
-  mach_port_deallocate (mach_task_self (), newright);
-  mach_port_deallocate (mach_task_self (), auth);
-
   newcred->hook = cred->hook;
-  
-  mutex_lock (&cred->po->cntl->lock);
   newcred->po = cred->po;
-  newcred->po->refcnt++;
-  mutex_unlock (&cred->po->cntl->lock);
-  
+  refcount_ref (&newcred->po->refcnt);
+
   do
     err = io_restrict_auth (newcred->po->cntl->underlying, &newcred->realnode,
-			    newcred->user->uids->ids, 
+			    newcred->user->uids->ids,
 			    newcred->user->uids->num,
 			    newcred->user->gids->ids,
 			    newcred->user->gids->num);
