@@ -1,8 +1,7 @@
 /* A translator for fifos
 
-   Copyright (C) 1995, 1996, 1997 Free Software Foundation, Inc.
-
-   Written by Miles Bader <miles@gnu.ai.mit.edu>
+   Copyright (C) 1995,96,97,98,2001,02 Free Software Foundation, Inc.
+   Written by Miles Bader <miles@gnu.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
@@ -19,6 +18,7 @@
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <errno.h>
 #include <unistd.h>
 #include <error.h>
@@ -26,7 +26,7 @@
 #include <fcntl.h>
 #include <argp.h>
 
-#include <cthreads.h>
+#include <pthread.h>
 #include <hurd.h>
 #include <hurd/ports.h>
 #include <hurd/trivfs.h>
@@ -35,6 +35,9 @@
 
 #include <version.h>
 
+#include "libtrivfs/trivfs_fs_S.h"
+#include "libtrivfs/trivfs_io_S.h"
+
 /* Global options.  These defaults are the standard ones, I think...   */
 int wait_for_reader = 1, wait_for_writer = 1;
 int one_reader = 1;
@@ -46,11 +49,11 @@ struct pipe_class *fifo_pipe_class;
 struct pipe *active_fifo = NULL;
 
 /* Lock this when changing ACTIVE_FIFO.  */
-struct mutex active_fifo_lock;
+pthread_mutex_t active_fifo_lock;
 /* Signal this when ACTIVE_FIFO may have changed.  */
-struct condition active_fifo_changed;
+pthread_cond_t active_fifo_changed;
 
-const char *argp_program_version = STANDARD_HURD_VERSION (null);
+const char *argp_program_version = STANDARD_HURD_VERSION (fifo);
 
 static struct argp_option options[] =
 {
@@ -74,10 +77,10 @@ parse_opt (int key, char *arg, struct argp_state *state)
 }
 
 static const struct argp argp = {
-  options, parse_opt, 0, "Translator for fifos"
+  options, parse_opt, 0, "Translator for fifos."
 };
 
-void
+int
 main (int argc, char **argv)
 {
   error_t err;
@@ -94,6 +97,7 @@ main (int argc, char **argv)
 
   /* Reply to our parent */
   err = trivfs_startup (bootstrap, 0, 0, 0, 0, 0, &fsys);
+  mach_port_deallocate (mach_task_self (), bootstrap);
   if (err)
     error (3, err, "Contacting parent");
 
@@ -101,12 +105,13 @@ main (int argc, char **argv)
   do
     {
       ports_enable_class (fsys->protid_class);
-      ports_manage_port_operations_multithread (fsys->pi.bucket, trivfs_demuxer,
-						30*1000, 5*60*1000, 0, 0);
+      ports_manage_port_operations_multithread (fsys->pi.bucket,
+						trivfs_demuxer,
+						30*1000, 5*60*1000, 0);
     }
   while (ports_count_class (fsys->protid_class) > 0);
 
-  exit(0);
+  return 0;
 }
 
 /* ---------------------------------------------------------------- */
@@ -119,18 +124,21 @@ open_hook (struct trivfs_peropen *po)
 
   if (flags & (O_READ | O_WRITE))
     {
-      mutex_lock (&active_fifo_lock);
+      pthread_mutex_lock (&active_fifo_lock);
 
 /* Wait until the active fifo has changed so that CONDITION is true.  */
-#define WAIT(condition, noblock_err)					     \
-  while (!err && !(condition)) 						     \
-    if (flags & O_NONBLOCK) 						     \
-      {			 						     \
-        err = noblock_err; 						     \
-        break;		 						     \
-      }			 						     \
-    else if (hurd_condition_wait (&active_fifo_changed, &active_fifo_lock))  \
-      err = EINTR;
+#define WAIT(condition, noblock_err)					      \
+  while (!err && !(condition))						      \
+    {									      \
+      if (flags & O_NONBLOCK)						      \
+	{								      \
+	  err = noblock_err;						      \
+	  break;							      \
+	}								      \
+      else if (pthread_hurd_cond_wait_np (&active_fifo_changed,		      \
+					  &active_fifo_lock))		      \
+	err = EINTR;							      \
+    }
 
       if (flags & O_READ)
 	/* When opening for read, what we do depends on what mode this server
@@ -156,12 +164,12 @@ open_hook (struct trivfs_peropen *po)
 	  if (!err)
 	    {
 	      pipe_add_reader (active_fifo);
-	      condition_broadcast (&active_fifo_changed);
+	      pthread_cond_broadcast (&active_fifo_changed);
 	      /* We'll unlock ACTIVE_FIFO_LOCK below; the writer code won't
 		 make us block because we've ensured that there's a reader
 		 for it.  */
 
-	      if (wait_for_writer)
+	      if (wait_for_writer && (!(flags & O_WRITE)))
 		/* Wait until there's a writer.  */
 		{
 		  WAIT (active_fifo->writers, 0);
@@ -170,7 +178,7 @@ open_hook (struct trivfs_peropen *po)
 		    {
 		      pipe_remove_reader (active_fifo);
 		      active_fifo = NULL;
-		      condition_broadcast (&active_fifo_changed);
+		      pthread_cond_broadcast (&active_fifo_changed);
 		    }
 		}
 	    }
@@ -195,13 +203,13 @@ open_hook (struct trivfs_peropen *po)
 	  if (!err)
 	    {
 	      pipe_add_writer (active_fifo);
-	      condition_broadcast (&active_fifo_changed);
+	      pthread_cond_broadcast (&active_fifo_changed);
 	    }
 	}
 
       po->hook = active_fifo;
 
-      mutex_unlock (&active_fifo_lock);
+      pthread_mutex_unlock (&active_fifo_lock);
     }
 
   return err;
@@ -217,7 +225,7 @@ close_hook (struct trivfs_peropen *po)
   if (!pipe)
     return;
 
-  mutex_lock (&active_fifo_lock);
+  pthread_mutex_lock (&active_fifo_lock);
   was_active = (active_fifo == pipe);
 
   if (was_active)
@@ -227,7 +235,7 @@ close_hook (struct trivfs_peropen *po)
 	|| ((flags & O_WRITE) && pipe->writers == 1);
   else
     /* Let others have their fun.  */
-    mutex_unlock (&active_fifo_lock);
+    pthread_mutex_unlock (&active_fifo_lock);
 
   if (flags & O_READ)
     pipe_remove_reader (pipe);
@@ -239,8 +247,8 @@ close_hook (struct trivfs_peropen *po)
     {
       if (detach)
 	active_fifo = NULL;
-      condition_broadcast (&active_fifo_changed);
-      mutex_unlock (&active_fifo_lock);
+      pthread_cond_broadcast (&active_fifo_changed);
+      pthread_mutex_unlock (&active_fifo_lock);
     }
 }
 
@@ -268,10 +276,10 @@ trivfs_modify_stat (struct trivfs_protid *cred, struct stat *st)
 
   if (pipe)
     {
-      mutex_lock (&pipe->lock);
+      pthread_mutex_lock (&pipe->lock);
       st->st_size = pipe_readable (pipe, 1);
       st->st_blocks = st->st_size >> 9;
-      mutex_unlock (&pipe->lock);
+      pthread_mutex_unlock (&pipe->lock);
     }
   else
     st->st_size = st->st_blocks = 0;
@@ -313,13 +321,14 @@ trivfs_goaway (struct trivfs_control *cntl, int flags)
    mapping; they will set none of the ports and return an error.  Such
    objects can still be accessed by io_read and io_write.  */
 error_t
-trivfs_S_io_map(struct trivfs_protid *cred,
-		memory_object_t *rdobj,
-		mach_msg_type_name_t *rdtype,
-		memory_object_t *wrobj,
-		mach_msg_type_name_t *wrtype)
+trivfs_S_io_map (struct trivfs_protid *cred,
+		 mach_port_t reply, mach_msg_type_name_t replytype,
+		 memory_object_t *rdobj,
+		 mach_msg_type_name_t *rdtype,
+		 memory_object_t *wrobj,
+		 mach_msg_type_name_t *wrtype)
 {
-  return EINVAL;
+  return EOPNOTSUPP;
 }
 
 /* ---------------------------------------------------------------- */
@@ -342,10 +351,10 @@ trivfs_S_io_read (struct trivfs_protid *cred,
   else
     {
       struct pipe *pipe = cred->po->hook;
-      mutex_lock (&pipe->lock);
+      pthread_mutex_lock (&pipe->lock);
       err = pipe_read (pipe, cred->po->openmodes & O_NONBLOCK, NULL,
 		       data, data_len, amount);
-      mutex_unlock (&pipe->lock);
+      pthread_mutex_unlock (&pipe->lock);
     }
 
   return err;
@@ -370,9 +379,9 @@ trivfs_S_io_readable (struct trivfs_protid *cred,
   else
     {
       struct pipe *pipe = cred->po->hook;
-      mutex_lock (&pipe->lock);
+      pthread_mutex_lock (&pipe->lock);
       *amount = pipe_readable (pipe, 1);
-      mutex_unlock (&pipe->lock);
+      pthread_mutex_unlock (&pipe->lock);
       err = 0;
     }
 
@@ -399,10 +408,10 @@ trivfs_S_io_seek (struct trivfs_protid *cred,
    return the types that are then available.  ID_TAG is returned as passed; it
    is just for the convenience of the user in matching up reply messages with
    specific requests sent.  */
-error_t
-trivfs_S_io_select (struct trivfs_protid *cred,
-		    mach_port_t reply, mach_msg_type_name_t reply_type,
-		    int *select_type, int *tag)
+static error_t
+io_select_common (struct trivfs_protid *cred,
+		  mach_port_t reply, mach_msg_type_name_t reply_type,
+		  struct timespec *tsp, int *select_type)
 {
   struct pipe *pipe;
   error_t err = 0;
@@ -414,26 +423,45 @@ trivfs_S_io_select (struct trivfs_protid *cred,
   pipe = cred->po->hook;
 
   if (*select_type & SELECT_READ)
-    if (cred->po->openmodes & O_READ)
-      {
-	mutex_lock (&pipe->lock);
-	if (pipe_wait_readable (pipe, 1, 1) != EWOULDBLOCK)
-	  ready |= SELECT_READ; /* Data immediately readable (or error). */
-	mutex_unlock (&pipe->lock);
-      }
-    else
-      ready |= SELECT_READ;	/* Error immediately available...  */
+    {
+      if (cred->po->openmodes & O_READ)
+	{
+	  pthread_mutex_lock (&pipe->lock);
+	  err = pipe_wait_readable (pipe, 1, 1);
+	  if (err == EWOULDBLOCK)
+	    err = 0; /* Not readable, actually not an error.  */
+	  else
+	    ready |= SELECT_READ; /* Data immediately readable (or error).  */
+	  pthread_mutex_unlock (&pipe->lock);
+	}
+      else
+	{
+	  err = EBADF;
+	  ready |= SELECT_READ;	/* Error immediately available...  */
+	}
+      if (err)
+	/* Prevent write test from overwriting err.  */
+	*select_type &= ~SELECT_WRITE;
+    }
 
   if (*select_type & SELECT_WRITE)
-    if (cred->po->openmodes & O_WRITE)
-      {
-	mutex_lock (&pipe->lock);
-	if (pipe_wait_writable (pipe, 1) != EWOULDBLOCK)
-	  ready |= SELECT_WRITE; /* Data immediately writable (or error). */
-	mutex_unlock (&pipe->lock);
-      }
-    else
-      ready |= SELECT_WRITE;	/* Error immediately available...  */
+    {
+      if (cred->po->openmodes & O_WRITE)
+	{
+	  pthread_mutex_lock (&pipe->lock);
+	  err = pipe_wait_writable (pipe, 1);
+	  if (err == EWOULDBLOCK)
+	    err = 0; /* Not writable, actually not an error.  */
+	  else
+	    ready |= SELECT_WRITE; /* Data immediately writable (or error).  */
+	  pthread_mutex_unlock (&pipe->lock);
+	}
+      else
+	{
+	  err = EBADF;
+	  ready |= SELECT_WRITE;	/* Error immediately available...  */
+	}
+    }
 
   if (ready)
     *select_type = ready;
@@ -441,11 +469,28 @@ trivfs_S_io_select (struct trivfs_protid *cred,
     /* Wait for something to change.  */
     {
       ports_interrupt_self_on_port_death (cred, reply);
-      err = pipe_pair_select (pipe, pipe, select_type, 1);
+      err = pipe_pair_select (pipe, pipe, tsp, select_type, 1);
     }
 
   return err;
 }
+
+error_t
+trivfs_S_io_select (struct trivfs_protid *cred,
+		    mach_port_t reply, mach_msg_type_name_t reply_type,
+		    int *select_type)
+{
+  return io_select_common (cred, reply, reply_type, NULL, select_type);
+}
+
+error_t
+trivfs_S_io_select_timeout (struct trivfs_protid *cred,
+			    mach_port_t reply, mach_msg_type_name_t reply_type,
+			    struct timespec ts,
+			    int *select_type)
+{
+  return io_select_common (cred, reply, reply_type, &ts, select_type);
+}
 
 /* ---------------------------------------------------------------- */
 
@@ -475,10 +520,10 @@ trivfs_S_io_write (struct trivfs_protid *cred,
 	err = EBADF;
       else
 	{
-	  mutex_lock (&pipe->lock);
+	  pthread_mutex_lock (&pipe->lock);
 	  err = pipe_write (pipe, flags & O_NONBLOCK, NULL,
 			    data, data_len, amount);
-	  mutex_unlock (&pipe->lock);
+	  pthread_mutex_unlock (&pipe->lock);
 	}
     }
 
@@ -498,7 +543,7 @@ trivfs_S_file_set_size (struct trivfs_protid *cred,
 /* These four routines modify the O_APPEND, O_ASYNC, O_FSYNC, and
    O_NONBLOCK bits for the IO object. In addition, io_get_openmodes
    will tell you which of O_READ, O_WRITE, and O_EXEC the object can
-   be used for.  The O_ASYNC bit affects icky async I/O; good async 
+   be used for.  The O_ASYNC bit affects icky async I/O; good async
    I/O is done through io_async which is orthogonal to these calls. */
 
 error_t
